@@ -23,6 +23,7 @@
 
 #include <glib/gi18n.h>
 #include "class_factory.hpp"
+#include "utils.h"
 
 #include "conf.h"
 
@@ -50,6 +51,8 @@ AppConf::AppConf() :
 	add_entry("/apps/stardict/preferences/dictionary/use_custom_font", false);
 #endif
 	add_entry("/apps/stardict/preferences/main_window/hide_on_startup", false);
+	add_entry("/apps/stardict/preferences/main_window/search_while_typing", true);
+	add_entry("/apps/stardict/preferences/main_window/showfirst_when_notfound", true);
 	add_entry("/apps/stardict/preferences/dictionary/enable_sound_event", true);
 	add_entry("/apps/stardict/preferences/main_window/hide_list", false);
 	add_entry("/apps/stardict/preferences/dictionary/scan_selection", true);
@@ -79,8 +82,18 @@ AppConf::AppConf() :
 #else
 	add_entry("/apps/stardict/preferences/dictionary/custom_font", std::string());
 #endif
+
+	add_entry("/apps/stardict/preferences/dictionary/create_cache_file", true);
+	add_entry("/apps/stardict/preferences/dictionary/enable_collation", false);
+	add_entry("/apps/stardict/preferences/dictionary/collate_function", 0);
+
 #if defined(CONFIG_GTK) || defined (CONFIG_GPE)
 	add_entry("/apps/stardict/preferences/dictionary/play_command", std::string("play"));
+#endif
+#ifdef _WIN32
+	add_entry("/apps/stardict/preferences/dictionary/tts_path", std::string("C:\\Program Files\\WyabdcRealPeopleTTS\nC:\\Program Files\\OtdRealPeopleTTS\nWyabdcRealPeopleTTS\nOtdRealPeopleTTS"));
+#else
+	add_entry("/apps/stardict/preferences/dictionary/tts_path", std::string("/usr/share/WyabdcRealPeopleTTS\n/usr/share/OtdRealPeopleTTS"));
 #endif
 	add_entry("/apps/stardict/preferences/dictionary/history", get_default_history_filename());
 	add_entry("/apps/stardict/preferences/dictionary/only_export_word", true);
@@ -193,12 +206,9 @@ std::string AppConf::get_default_history_filename()
 {
 std::string histname;
 #ifdef _WIN32
-  histname=gStarDictDataDir + G_DIR_SEPARATOR_S "history.txt";
+	histname = get_user_config_dir() + G_DIR_SEPARATOR_S "history.txt";
 #else
-  gchar *datadir = g_build_filename(g_get_home_dir(), ".stardict", NULL);
-  histname=datadir;
-  g_free(datadir);
-  histname+= G_DIR_SEPARATOR_S "history";
+	histname = get_user_config_dir() + G_DIR_SEPARATOR_S "history";
 #endif
 
   return histname;
