@@ -12,9 +12,9 @@
 #endif
 
 #include "conf.h"
-#include "lib.h"
+#include "lib/common.hpp"
 #include "utils.h"
-#include "file.hpp"
+#include "lib/file.hpp"
 
 #include "dictmanagedlg.h"
 
@@ -60,7 +60,6 @@ public:
 												 6, dictinfo.description.c_str(), 
 												 7, dictinfo.date.c_str(), 
 												 8, url.c_str(), 
-												 9, TRUE, //for editable.
 												 -1);
 		}
 	}
@@ -74,10 +73,9 @@ static GtkTreeModel* __create_dict_tree_model(const std::list<std::string> &dirs
 																							const std::list<std::string>& disable_list, bool istreedict)
 {
 	GtkListStore *model = 
-		gtk_list_store_new(10, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_LONG, 
-											 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
-											 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
-											 G_TYPE_BOOLEAN);
+		gtk_list_store_new(9, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_LONG, 
+					 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+					 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 	for_each_file(dirs, ".ifo", order_list, disable_list, 
 								GetInfo(model, istreedict));
 	
@@ -234,6 +232,46 @@ void DictManageDlg::drag_data_received_cb(GtkWidget *widget, GdkDragContext *ctx
 	}
 }
 
+gboolean DictManageDlg::on_treeview_button_press(GtkWidget * widget, GdkEventButton * event, DictManageDlg *oDictManageDlg)
+{
+	if (event->type==GDK_2BUTTON_PRESS) {
+		GtkTreeModel *model;
+		GtkTreeIter iter;
+
+		GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
+		if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+			gchar *dictname, *author, *email, *website, *description, *date, *filename;
+			glong wordcount;
+			gtk_tree_model_get (model, &iter, 1, &dictname, 2, &wordcount, 3, &author, 4, &email, 5, &website, 6, &description, 7, &date, 8, &filename, -1);
+			GtkWidget *dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(oDictManageDlg->window),
+				GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_MESSAGE_INFO,
+				GTK_BUTTONS_OK,
+				"<b>%s:</b> %s\n<b>%s:</b> %ld\n<b>%s:</b> %s\n<b>%s:</b> %s\n<b>%s:</b> %s\n<b>%s:</b> %s\n<b>%s:</b> %s\n<b>%s:</b> %s",
+				_("Dictionary Name"), dictname,
+				_("Word count"), wordcount,
+				_("Author"), author,
+				_("Email"), email,
+				_("Website"), website,
+				_("Description"), description,
+				_("Date"), date,
+				_("File name"), filename);
+			g_free(dictname);
+			g_free(author);
+			g_free(email);
+			g_free(website);
+			g_free(description);
+			g_free(date);
+			g_free(filename);
+			if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_NONE)
+				gtk_widget_destroy (dialog);
+		}
+		return true;
+	} else {
+		return false;
+	}
+}
+
 GtkWidget *DictManageDlg::create_dict_tree(gboolean istreedict)
 {	
 	GtkWidget *sw;	
@@ -252,6 +290,8 @@ GtkWidget *DictManageDlg::create_dict_tree(gboolean istreedict)
 		dict_tree_model = now_tree_model;
 
 	GtkWidget *now_treeview = gtk_tree_view_new_with_model (now_tree_model);
+	g_signal_connect(G_OBJECT(now_treeview), "button_press_event",
+                   G_CALLBACK(on_treeview_button_press), this);
 	if (istreedict)
 		treedict_treeview = now_treeview;
 	else
@@ -288,7 +328,7 @@ GtkWidget *DictManageDlg::create_dict_tree(gboolean istreedict)
 	gtk_tree_view_append_column (GTK_TREE_VIEW(now_treeview), column);
   	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);
 
-	renderer = gtk_cell_renderer_text_new ();
+	/*renderer = gtk_cell_renderer_text_new ();
 	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);  
 	column = gtk_tree_view_column_new_with_attributes (_("Author"), renderer, "text", 3, "editable", 9, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW(now_treeview), column);
@@ -322,7 +362,7 @@ GtkWidget *DictManageDlg::create_dict_tree(gboolean istreedict)
 	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);  
 	column = gtk_tree_view_column_new_with_attributes (_("File name"), renderer, "text", 8, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW(now_treeview), column);
-  	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);
+  	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);*/
 
 	GtkTargetEntry gte[] = {{"STARDICT_DICTMANAGE", GTK_TARGET_SAME_APP, 0}};
 	gtk_tree_view_enable_model_drag_source(GTK_TREE_VIEW(now_treeview), GDK_BUTTON1_MASK, gte, 1, GDK_ACTION_COPY);
@@ -504,7 +544,7 @@ GtkWidget *DictManageDlg::create_buttons()
 	return vbox;
 }
 
-void DictManageDlg::Show()
+bool DictManageDlg::Show()
 {
 	if (!window) {	
 		window = gtk_dialog_new();
@@ -593,9 +633,16 @@ void DictManageDlg::Show()
 	
 		gtk_window_set_title(GTK_WINDOW (window), _("Manage Dictionaries"));	
 	}
-	while (gtk_dialog_run(GTK_DIALOG(window))==GTK_RESPONSE_HELP)
-    ;
-	gtk_widget_hide(GTK_WIDGET(window));
+	gint result;
+	while ((result = gtk_dialog_run(GTK_DIALOG(window)))==GTK_RESPONSE_HELP)
+		;
+	if (result == GTK_RESPONSE_NONE) {
+		// Caused by gtk_widget_destroy(), quiting.
+		return true;
+	} else {
+		gtk_widget_hide(GTK_WIDGET(window));
+		return false;
+	}
 }
 
 void DictManageDlg::Close()
