@@ -25,6 +25,7 @@ private:
 	static void on_wazard_button_toggled(GtkToggleButton *button, DictManageDlg *oDictManageDlg);
 	static void on_appendix_button_toggled(GtkToggleButton *button, DictManageDlg *oDictManageDlg);
 
+	static gboolean on_treeview_button_press(GtkWidget * widget, GdkEventButton *event, DictManageDlg *oDictManageDlg);
 	static void response_handler (GtkDialog *dialog, gint res_id, DictManageDlg *oDictManageDlg);
 	static void on_move_top_button_clicked(GtkWidget *widget, DictManageDlg *oDictManageDlg);
 	static void on_move_bottom_button_clicked(GtkWidget *widget, DictManageDlg *oDictManageDlg);
@@ -37,7 +38,7 @@ private:
 	static void drag_data_get_cb(GtkWidget *widget, GdkDragContext *ctx, GtkSelectionData *data, guint info, guint time, DictManageDlg *oDictManageDlg);
 public:
 	DictManageDlg(GtkWindow *parent_win, GdkPixbuf *dicts_icon, GdkPixbuf *tree_dicts_icon);
-	void Show();
+	bool Show();
 	void Close();
 };
 
