@@ -26,13 +26,17 @@ DockLet::DockLet()
 void DockLet::Create(DockLetIconType iconType)
 {	
 	docklet = egg_tray_icon_new("StarDict");
-	GtkWidget *box = gtk_event_box_new();
-	if (iconType == DOCKLET_NORMAL_ICON)
+	box = gtk_event_box_new();
+	if (iconType == DOCKLET_NORMAL_ICON) {
+		gtk_tooltips_set_tip(gpAppFrame->tooltips, box,_("StarDict"),NULL);
 		image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.docklet_normal_icon.get());
-	else if (iconType == DOCKLET_SCAN_ICON)
+	} else if (iconType == DOCKLET_SCAN_ICON) {
+		gtk_tooltips_set_tip(gpAppFrame->tooltips, box,_("StarDict - Scanning"),NULL);
 		image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.docklet_scan_icon.get());
-	else
+	} else {
+		gtk_tooltips_set_tip(gpAppFrame->tooltips, box,_("StarDict - Stopped"),NULL);
 		image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.docklet_stop_icon.get());
+	}
 	current_icon = iconType;
 
 	g_signal_connect(G_OBJECT(docklet), "embedded", G_CALLBACK(EmbeddedCallback), NULL);
@@ -55,17 +59,18 @@ void DockLet::SetIcon(DockLetIconType icon_type)
 		return;
 	GdkPixbuf *p;
 	switch (icon_type) {
-		case DOCKLET_NORMAL_ICON:
-			p = gpAppFrame->oAppSkin.docklet_normal_icon.get();
-			break;
 		case DOCKLET_SCAN_ICON:
+			gtk_tooltips_set_tip(gpAppFrame->tooltips, box,_("StarDict - Scanning"),NULL);
 			p = gpAppFrame->oAppSkin.docklet_scan_icon.get();
 			break;
 		case DOCKLET_STOP_ICON:
+			gtk_tooltips_set_tip(gpAppFrame->tooltips, box,_("StarDict - Stopped"),NULL);
 			p = gpAppFrame->oAppSkin.docklet_stop_icon.get();
 			break;
 		default:
+			gtk_tooltips_set_tip(gpAppFrame->tooltips, box,_("StarDict"),NULL);
 			p = gpAppFrame->oAppSkin.docklet_normal_icon.get();
+			break;
 	}
 	gtk_image_set_from_pixbuf(GTK_IMAGE(image),p);
 	current_icon = icon_type;
