@@ -13,6 +13,7 @@ enum DockLetIconType {
 class DockLet{
 private:
 	EggTrayIcon *docklet;
+	GtkWidget *box;
 	GtkWidget *image; //icon image.
 	GtkWidget *menu,*scan_menuitem;
 	DockLetIconType current_icon;
@@ -29,7 +30,7 @@ private:
 	void PopupMenu(GdkEventButton *event);
 public:	
 	gboolean embedded;
-	gboolean query_in_floatwin;
+
 	DockLet();
 	void Create(DockLetIconType iconType = DOCKLET_NORMAL_ICON);
 	void End();
