@@ -60,8 +60,8 @@ public:
   FloatWin();
   void Create();
   void End();
-  void ShowText(const gchar **ppWord, gchar ** ppWordData, const gchar * sOriginWord);
-  void ShowText(gchar ***pppWord, gchar *** pppWordData, const gchar ** ppOriginWord, gint count,const gchar * sOriginWord);
+  void ShowText(gchar ***pppWord, gchar **** ppppWordData, const gchar * sOriginWord);
+  void ShowText(gchar ****ppppWord, gchar ***** pppppWordData, const gchar ** ppOriginWord, gint count,const gchar * sOriginWord);
   void ShowNotFound(const char* sWord,const char* sReason, gboolean fuzzy);
   void Show();
   void Hide();
