@@ -21,12 +21,12 @@
 #  include "config.h"
 #endif
 
-#include <sys/stat.h>
+#include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 
 #ifdef _WIN32
-#define VERSION "2.4.7"
+#define VERSION "2.4.8"
 #  include <gdk/gdkwin32.h>
 #endif
 
@@ -155,6 +155,7 @@ void TopWin::Create(GtkWidget *vbox)
 
 void TopWin::Destroy(void)
 {
+	InsertHisList(GetText());
 	SaveHistory();
 	if (MainMenu)
 		gtk_widget_destroy(MainMenu);
@@ -168,11 +169,14 @@ void TopWin::on_entry_changed(GtkEntry *entry, TopWin *oTopWin)
 {
 	if (!(oTopWin->enable_change_cb))
 		return;
+	if (!conf->get_bool("/apps/stardict/preferences/main_window/search_while_typing"))
+		return;
 	const gchar *sWord = gtk_entry_get_text(entry);
 	if(sWord[0]!='\0') {
 		gpAppFrame->TopWinWordChange(sWord);
 	} else {
 		gpAppFrame->oMidWin.oTextWin.queryWord.clear();
+		gpAppFrame->oMidWin.oIndexWin.oResultWin.Clear();
 		gpAppFrame->oMidWin.oTextWin.ShowTips();
 	}
 }
@@ -326,20 +330,17 @@ void TopWin::do_previous()
 		}
 		else {
 			// user have selected the first row.
-			const gchar *preword = gpAppFrame->oLibs.poGetPreWord(gpAppFrame->iCurrentIndex);
+			gchar *word;
+			gtk_tree_model_get (model, &iter, 0, &word, -1);
+			CurrentIndex *iPreIndex = (CurrentIndex *)g_malloc(sizeof(CurrentIndex) * gpAppFrame->oLibs.ndicts());
+			const gchar *preword = gpAppFrame->oLibs.poGetPreWord(word, iPreIndex);
 			if (preword) {
-				SetText_without_notify(preword);
+				SetText(preword);
 				if (GTK_WIDGET_HAS_FOCUS(GTK_WIDGET(GTK_COMBO(WordCombo)->entry)))
 					gtk_editable_select_region(GTK_EDITABLE(GTK_COMBO(WordCombo)->entry),0,-1);
-				gpAppFrame->SimpleLookupToTextWin(preword,gpAppFrame->iCurrentIndex,true);
-				
-				gpAppFrame->ListWords(gpAppFrame->iCurrentIndex);
-				// the next lines should be faster than ListWords(),but as the TreeView can't be freezed,it make it interface changing not so smoothly.
-				/*gpAppFrame->oMidWin.oListWin.Prepend(preword);
-				gpAppFrame->oMidWin.oListWin.RemoveLast();
-				gpAppFrame->oMidWin.oListWin.UnSelectAll();
-        			gpAppFrame->oMidWin.oListWin.ReScroll();*/
 			}
+			g_free(iPreIndex);
+			g_free(word);
 		}		
 		gtk_tree_path_free(path);
 	}
@@ -405,14 +406,12 @@ void TopWin::do_next()
 		else {
 			// user have selected the last row.
 			gtk_tree_model_get (model, &new_iter, 0, &word, -1);
-			glong *iNextIndex = (glong *)g_malloc(sizeof(glong) * gpAppFrame->oLibs.ndicts());
+			CurrentIndex *iNextIndex = (CurrentIndex *)g_malloc(sizeof(CurrentIndex) * gpAppFrame->oLibs.ndicts());
 			const gchar *nextword = gpAppFrame->oLibs.poGetNextWord(word, iNextIndex);
 			if (nextword) {
-				SetText_without_notify(nextword);
+				SetText(nextword);
 				if (GTK_WIDGET_HAS_FOCUS(GTK_WIDGET(GTK_COMBO(WordCombo)->entry)))
 					gtk_editable_select_region(GTK_EDITABLE(GTK_COMBO(WordCombo)->entry),0,-1);
-				gpAppFrame->SimpleLookupToTextWin(nextword,iNextIndex,true);
-				gpAppFrame->ListWords(iNextIndex);
 			}
 			g_free(iNextIndex);
 			g_free(word);
@@ -566,22 +565,31 @@ void TopWin::MenuCallback(GtkWidget *widget, TopWin *oTopWin)
 	oTopWin->do_menu();
 }
 
-void TopWin::SetText(const gchar *word)
+void TopWin::SetText(const gchar *word, bool notify)
 {
 	if (strcmp(word, gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(WordCombo)->entry))) == 0)
 		return;
 	enable_change_cb = false;
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(WordCombo)->entry),word); // this will emit "changed" signal twice, one for "", one for word. so disable it.
 	enable_change_cb = true;
-	on_entry_changed(GTK_ENTRY(GTK_COMBO(WordCombo)->entry), this);
+
+	if (!notify)
+		return;
+	if(word[0]!='\0') {
+		gpAppFrame->TopWinWordChange(word);
+	} else {
+		gpAppFrame->oMidWin.oTextWin.queryWord.clear();
+		gpAppFrame->oMidWin.oIndexWin.oResultWin.Clear();
+		gpAppFrame->oMidWin.oTextWin.ShowTips();
+	}
 }
 
-void TopWin::SetText_without_notify(const gchar *word)
+/*void TopWin::SetText_without_notify(const gchar *word)
 {
 	enable_change_cb = false;
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(WordCombo)->entry),word);
 	enable_change_cb = true;
-}
+}*/
 
 const gchar* TopWin::GetText()
 {
@@ -648,7 +656,7 @@ void TopWin::InsertHisList(const gchar *word)
 
 void TopWin::SaveHistory()
 {
-	FILE *f=fopen(conf->get_string("/apps/stardict/preferences/dictionary/history").c_str(), "w");
+	FILE *f=g_fopen(conf->get_string("/apps/stardict/preferences/dictionary/history").c_str(), "w");
 	if (!f)
 		return;
 	GList *hist_list = HisList;
@@ -663,10 +671,10 @@ void TopWin::LoadHistory()
 {
 	const gchar *filename = conf->get_string("/apps/stardict/preferences/dictionary/history").c_str();
 	struct stat stats;
-	if (stat (filename, &stats) == -1)
+	if (g_stat (filename, &stats) == -1)
         	return;
 	FILE *historyfile;
-	historyfile = fopen(filename,"r");
+	historyfile = g_fopen(filename,"r");
 	if (!historyfile)
 		return;
 	gchar *buffer = (gchar *)g_malloc (stats.st_size + 1);
@@ -837,7 +845,6 @@ void ListWin::InsertLast(const gchar *word)
 	gtk_list_store_set (list_model, &iter, 0, word, -1);
 }
 
-
 void ListWin::SetTreeModel(std::vector<gchar *> *reslist)
 {
 	GtkTreeIter parent;
@@ -894,22 +901,17 @@ void ListWin::on_selection_changed(GtkTreeSelection *selection, ListWin *oListWi
 	
 	if (gtk_tree_selection_get_selected (selection, &model, &iter))
 	{
-		gchar *path_str = gtk_tree_model_get_string_from_iter(model,&iter);
 		if (!oListWin->nowIsList) {
+			gchar *path_str = gtk_tree_model_get_string_from_iter(model,&iter);
 			if (!strchr(path_str, ':')) { // bookname entry.
 				g_free(path_str);
 				return;
 			}
+			g_free(path_str);
 		}
 		gchar *word;
 		gtk_tree_model_get (model, &iter, 0, &word, -1);
-		// the "0" is the first iter's string.when the user select some row while the list have no selection,GTK will select
-		// the first row,then select the line that user clicked.so here cache the first line is not bad :_)
-		if (!strcmp(path_str,"0"))
-			gpAppFrame->SimpleLookupToTextWin(word,gpAppFrame->iCurrentIndex,true); //the first word's index is already cached.
-		else
-			gpAppFrame->SimpleLookupToTextWin(word,NULL);
-		g_free(path_str);
+		gpAppFrame->SimpleLookupToTextWin(word,NULL);
 		g_free(word);
 	}
 }
@@ -1015,6 +1017,67 @@ void TreeWin::on_selection_changed(GtkTreeSelection *selection, TreeWin *oTreeWi
 }
 
 
+/**************************************************/
+void ResultWin::Create(GtkWidget *notebook)
+{
+	GtkListStore *model = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+	treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL(model));
+	gtk_widget_show(treeview);
+	g_object_unref (model);
+	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
+	GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
+	GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes ("word", renderer,
+						     "text", 0, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+
+	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+	g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (on_selection_changed), this);
+	
+	GtkWidget *scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_show(scrolledwindow);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow),
+				       GTK_SHADOW_ETCHED_IN);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
+				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+		
+	gtk_container_add(GTK_CONTAINER(scrolledwindow),treeview);
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), scrolledwindow, NULL);
+}
+
+void ResultWin::InsertLast(const gchar *word, const gchar *mark)
+{
+	GtkListStore *model = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(treeview)));
+	GtkTreeIter iter;
+	gtk_list_store_append (model, &iter);
+	gtk_list_store_set (model, &iter, 0, word, 1, mark, -1);
+}
+
+void ResultWin::Clear()
+{
+	GtkListStore *model = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(treeview)));
+	gtk_list_store_clear(model);
+}
+
+void ResultWin::on_selection_changed(GtkTreeSelection *selection, ResultWin *oResultWin)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	
+	if (gtk_tree_selection_get_selected (selection, &model, &iter))
+	{
+		gchar *markstr;
+		gtk_tree_model_get(model, &iter, 1, &markstr, -1);
+		GtkTextView *textview = GTK_TEXT_VIEW(gpAppFrame->oMidWin.oTextWin.view->Widget());
+		GtkTextBuffer *buffer = gtk_text_view_get_buffer(textview);
+		GtkTextMark *mark = gtk_text_buffer_get_mark(buffer, markstr);
+		g_free(markstr);
+		if (mark) {
+			gtk_text_view_scroll_to_mark(textview, mark, 0.0, TRUE, 0.0, 0.0);
+		}
+	}
+}
+
 /**************************************************/
 IndexWin::IndexWin()
 {
@@ -1033,15 +1096,16 @@ void IndexWin::Create(GtkWidget *hpaned)
 	gtk_box_pack_start(GTK_BOX(vbox),notebook, true, true, 0);
 	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), false);
 	oListWin.Create(notebook);
-	if (!oTreeWin.Create(notebook))
-		return;
-	GtkWidget *hbox = gtk_hbox_new(true, 3);
-	gtk_box_pack_start(GTK_BOX(vbox),hbox, false, false, 0);
+	oResultWin.Create(notebook);
+
+	GtkWidget *table = gtk_table_new(2, 2, FALSE);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 3);
+	gtk_box_pack_start(GTK_BOX(vbox),table, false, false, 0);
 	
 	GtkWidget *wazard_button = gtk_radio_button_new(NULL);
 	GTK_WIDGET_UNSET_FLAGS (wazard_button, GTK_CAN_FOCUS);
 	gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(wazard_button), false);
-	gtk_box_pack_start (GTK_BOX (hbox), wazard_button, true, true, 0);	
+	gtk_table_attach(GTK_TABLE(table), wazard_button, 0, 1, 0, 1, (GtkAttachOptions)(GTK_EXPAND|GTK_SHRINK|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_SHRINK|GTK_FILL), 0, 0);
 #ifdef CONFIG_GPE
 	GtkWidget *image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.index_wazard.get());
 	gtk_container_add (GTK_CONTAINER (wazard_button), image);
@@ -1058,25 +1122,46 @@ void IndexWin::Create(GtkWidget *hpaned)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wazard_button), true);
 	g_signal_connect(G_OBJECT(wazard_button),"toggled", G_CALLBACK(on_wazard_button_toggled), this);
 
-	GtkWidget *appendix_button = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(wazard_button));
-	GTK_WIDGET_UNSET_FLAGS (appendix_button, GTK_CAN_FOCUS);
-	gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(appendix_button), false);
-	gtk_box_pack_start (GTK_BOX (hbox), appendix_button, true, true, 0);	
+	GtkWidget *result_button = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(wazard_button));
+	GTK_WIDGET_UNSET_FLAGS (result_button, GTK_CAN_FOCUS);
+	gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(result_button), false);
+	gtk_table_attach(GTK_TABLE(table), result_button, 1, 2, 0, 1, (GtkAttachOptions)(GTK_EXPAND|GTK_SHRINK|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_SHRINK|GTK_FILL), 0, 0);
 #ifdef CONFIG_GPE
-	image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.index_appendix.get());
-	gtk_container_add (GTK_CONTAINER (appendix_button), image);
+	image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.index_dictlist.get());
+	gtk_container_add (GTK_CONTAINER (result_button), image);
 #else
 	hbox1 = gtk_hbox_new(false, 2);
-	gtk_container_add (GTK_CONTAINER (appendix_button), hbox1);
-	image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.index_appendix.get());
+	gtk_container_add (GTK_CONTAINER (result_button), hbox1);
+	image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.index_dictlist.get());
 	gtk_box_pack_start (GTK_BOX (hbox1), image, FALSE, FALSE, 0);
-	label = gtk_label_new_with_mnemonic(_("_Tree"));
+	label = gtk_label_new_with_mnemonic(_("_Result"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label), appendix_button);	
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), result_button);
 #endif
-	g_signal_connect(G_OBJECT(appendix_button),"toggled", G_CALLBACK(on_appendix_button_toggled), this);	
-	gtk_widget_show_all(hbox);
+	g_signal_connect(G_OBJECT(result_button),"toggled", G_CALLBACK(on_result_button_toggled), this);
+
+	if (oTreeWin.Create(notebook)) {
+		GtkWidget *appendix_button = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(wazard_button));
+		GTK_WIDGET_UNSET_FLAGS (appendix_button, GTK_CAN_FOCUS);
+		gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(appendix_button), false);
+		gtk_table_attach(GTK_TABLE(table), appendix_button, 0, 2, 1, 2, (GtkAttachOptions)(GTK_EXPAND|GTK_SHRINK|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_SHRINK|GTK_FILL), 0, 0);
+#ifdef CONFIG_GPE
+		image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.index_appendix.get());
+		gtk_container_add (GTK_CONTAINER (appendix_button), image);
+#else
+		hbox1 = gtk_hbox_new(false, 2);
+		gtk_container_add (GTK_CONTAINER (appendix_button), hbox1);
+		image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.index_appendix.get());
+		gtk_box_pack_start (GTK_BOX (hbox1), image, FALSE, FALSE, 0);
+		label = gtk_label_new_with_mnemonic(_("_Tree"));
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+		gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
+		gtk_label_set_mnemonic_widget(GTK_LABEL(label), appendix_button);	
+#endif
+		g_signal_connect(G_OBJECT(appendix_button),"toggled", G_CALLBACK(on_appendix_button_toggled), this);
+	}
+	gtk_widget_show_all(table);
 }
 
 void IndexWin::on_wazard_button_toggled(GtkToggleButton *button, IndexWin *oIndexWin)
@@ -1085,12 +1170,18 @@ void IndexWin::on_wazard_button_toggled(GtkToggleButton *button, IndexWin *oInde
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(oIndexWin->notebook), 0);
 }
 
-void IndexWin::on_appendix_button_toggled(GtkToggleButton *button, IndexWin *oIndexWin)
+void IndexWin::on_result_button_toggled(GtkToggleButton *button, IndexWin *oIndexWin)
 {
 	if (gtk_toggle_button_get_active(button))
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(oIndexWin->notebook), 1);
 }
 
+void IndexWin::on_appendix_button_toggled(GtkToggleButton *button, IndexWin *oIndexWin)
+{
+	if (gtk_toggle_button_get_active(button))
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(oIndexWin->notebook), 2);
+}
+
 /************************************************/
 
 void ToolWin::Create(GtkWidget *vbox)
@@ -1443,9 +1534,6 @@ void TextWin::ShowInfo()
 		  "       Statement: This program is distributed in the hope that it will be useful, "
 		  "but WITHOUT ANY WARRANTY; without the warranty "
 		  "that the word spelling, definition, and phonetic information are correct.\n"
-		  "\n"
-		  "StarDict 2.4.7 released on 2006.4.18\n"
-		  "http://stardict.sourceforge.net\n"		
 		  )
 		);
 
@@ -1458,29 +1546,45 @@ void TextWin::Show(const gchar *str)
   view->ScrollTo(0);
 }
 
-void TextWin::Show(const gchar **Word, gchar **WordData, const gchar * sOriginWord)
+void TextWin::Show(gchar ***Word, gchar ****WordData)
 {
-  view->BeginUpdate();
-  view->Clear();
-  view->GotoBegin();
+	view->BeginUpdate();
+	view->Clear();
+	view->GotoBegin();
 
-  for (int i=0; i<gpAppFrame->oLibs.ndicts(); i++) {
-    if (Word[i])
-      view->AppendHeader(gpAppFrame->oLibs.dict_name(i), Word[i]);
-    if (WordData[i])
-      view->AppendData(WordData[i]);
-  }
-  view->EndUpdate();
+	int j,k;
+	for (int i=0; i<gpAppFrame->oLibs.ndicts(); i++) {
+		if (Word[i]) {
+			view->AppendHeader(gpAppFrame->oLibs.dict_name(i), i);
+			j=0;
+			do {
+				view->AppendWord(Word[i][j]);
+				view->AppendData(WordData[i][j][0], Word[i][j]);
+				view->AppendNewline();
+				k=1;
+				while (WordData[i][j][k]) {
+					view->AppendDataSeparate();
+					view->AppendData(WordData[i][j][k], Word[i][j]);
+					view->AppendNewline();
+					k++;
+				}
+				j++;
+			} while (Word[i][j]);
+		}
+	}
+	view->EndUpdate();
 }
 
-void TextWin::ShowTreeDictData(const gchar *data)
+void TextWin::ShowTreeDictData(gchar *data)
 {
-  view->BeginUpdate();
-  view->Clear();
-  view->GotoBegin();
-  if (data)
-    view->AppendData(data);
-  view->EndUpdate();
+	view->BeginUpdate();
+	view->Clear();
+	view->GotoBegin();
+	if (data) {
+		view->AppendData(data, "");
+		view->AppendNewline();
+	}
+	view->EndUpdate();
 }
 
 gboolean TextWin::Find (const gchar *text, gboolean start)
@@ -1527,8 +1631,8 @@ gboolean TextWin::Find (const gchar *text, gboolean start)
     
     gtk_window_set_resizable (GTK_WINDOW (message_dlg), FALSE);
     
-    gtk_dialog_run(GTK_DIALOG(message_dlg));
-    gtk_widget_destroy(message_dlg);
+    if (gtk_dialog_run(GTK_DIALOG(message_dlg))!=GTK_RESPONSE_NONE)
+      gtk_widget_destroy(message_dlg);
     return FALSE;
   }
 }
