@@ -17,6 +17,7 @@ enum TextWinQueryResult
 {
 	TEXT_WIN_FOUND,
 	TEXT_WIN_NOT_FOUND,
+	TEXT_WIN_SHOW_FIRST,
 	TEXT_WIN_FUZZY_NOT_FOUND,
 	TEXT_WIN_PATTERN_NOT_FOUND,
 	TEXT_WIN_TIPS, // when the word entry is empty,it will show tip
@@ -80,8 +81,8 @@ public:
 
 	void Create(GtkWidget *vbox);
 	void Destroy(void);
-	void SetText(const gchar *word);
-	void SetText_without_notify(const gchar *word);
+	void SetText(const gchar *word, bool notify=true);
+	//void SetText_without_notify(const gchar *word);
 	const gchar* GetText();
 	void TextSelectAll();
 	void GrabFocus();
@@ -133,12 +134,23 @@ private:
 	static void on_selection_changed(GtkTreeSelection *selection, TreeWin *oTreeWin);
 };
 
+class ResultWin {
+public:
+	GtkWidget *treeview;
+	void Create(GtkWidget *notebook);
+	void InsertLast(const gchar *word, const gchar *mark);
+	void Clear();
+private:
+	static void on_selection_changed(GtkTreeSelection *selection, ResultWin *oResultWin);
+};
+
 class IndexWin {
 public:
 	GtkWidget *vbox;
 	GtkWidget *notebook;
 
 	ListWin oListWin;
+	ResultWin oResultWin;
 	TreeWin oTreeWin;
 
 	IndexWin();
@@ -146,6 +158,7 @@ public:
 private:
 	static void on_wazard_button_toggled(GtkToggleButton *button, IndexWin *oIndexWin);
 	static void on_appendix_button_toggled(GtkToggleButton *button, IndexWin *oIndexWin);
+	static void on_result_button_toggled(GtkToggleButton *button, IndexWin *oIndexWin);
 };
 
 class ToolWin {
@@ -188,8 +201,8 @@ public:
   void ShowInfo();
   void ShowInitFailed();
   void Show(const gchar *str);
-  void Show(const gchar **Word, gchar **WordData, const gchar * sOriginWord);
-  void ShowTreeDictData(const gchar *data);
+  void Show(gchar ***Word, gchar ****WordData);
+  void ShowTreeDictData(gchar *data);
   gboolean Find (const gchar *text, gboolean start);
 	bool IsSearchPanelHasFocus() { return GTK_WIDGET_HAS_FOCUS(eSearch); } 
 
