@@ -1,6 +1,7 @@
 #ifndef PANGOVIEW_H
 #define PAGNOVIEW_H
 
+#include <list>
 #include <string>
 #include <gtk/gtk.h>
 
@@ -12,6 +13,7 @@ public:
   void SetText(const char *str);
   void AppendPangoText(const char *str);
   void SetPangoText(const char *str); 
+  void AppendMark(const char *mark);
 
   GtkWidget *Widget(void) {
     if (autoresize)
@@ -44,6 +46,7 @@ private:
   bool autoresize;
   GtkScrolledWindow *scrolled_window;
   GtkTextIter iter;
+  std::list<GtkTextMark *> marklist;
   union {
     GtkTextView *textview;
     GtkLabel *label;
