@@ -22,12 +22,13 @@
 #endif
 
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 
 #ifdef _WIN32
 #  include <gdk/gdkwin32.h>
-#  include "stardict.h"
 #endif
 
+#include "stardict.h"
 #include "conf.h"
 #include "utils.h"
 #include "iskeyspressed.hpp"
@@ -35,16 +36,18 @@
 #include "prefsdlg.h"
 
 #ifndef CONFIG_GPE
-#define LOGO										0
+#define LOGO								0
 #define	DICTIONARY_SCAN_SETTINGS 					1
 #define	DICTIONARY_FONT_SETTINGS 					2
-#define DICTIONARY_EXPORT_SETTINGS					3
-#define	DICTIONARY_SOUND_SETTINGS 					4
-#define	MAINWIN_OPTIONS_SETTINGS 					5
-#define MAINWIN_SEARCH_WEBSITE_SETTINGS					6
-#define NOTIFICATION_AREA_ICON_OPITIONS_SETTINGS			7
-#define FLOATWIN_OPTIONS_SETTINGS					8
-#define FLOATWIN_SIZE_SETTINGS						9
+#define DICTIONARY_CACHE_SETTINGS					3
+#define DICTIONARY_EXPORT_SETTINGS					4
+#define	DICTIONARY_SOUND_SETTINGS 					5
+#define	MAINWIN_INPUT_SETTINGS 						6
+#define	MAINWIN_OPTIONS_SETTINGS 					7
+#define MAINWIN_SEARCH_WEBSITE_SETTINGS					8
+#define NOTIFICATION_AREA_ICON_OPITIONS_SETTINGS			9
+#define FLOATWIN_OPTIONS_SETTINGS					10
+#define FLOATWIN_SIZE_SETTINGS						11
 
 enum
 {
@@ -66,6 +69,7 @@ static CategoriesTreeItem dictionary_behavior [] =
 {
 	{N_("Scan Selection"), NULL, DICTIONARY_SCAN_SETTINGS},
 	{N_("Font"), NULL, DICTIONARY_FONT_SETTINGS},
+	{N_("Cache"), NULL, DICTIONARY_CACHE_SETTINGS},
 	{N_("Export"), NULL, DICTIONARY_EXPORT_SETTINGS},
 	{N_("Sound"), NULL, DICTIONARY_SOUND_SETTINGS},
 	
@@ -74,6 +78,7 @@ static CategoriesTreeItem dictionary_behavior [] =
 
 static CategoriesTreeItem mainwin_behavior [] =
 {
+	{N_("Input"), NULL, MAINWIN_INPUT_SETTINGS},
 	{N_("Options"), NULL, MAINWIN_OPTIONS_SETTINGS},
 	{N_("Search website"), NULL, MAINWIN_SEARCH_WEBSITE_SETTINGS},
 	
@@ -518,6 +523,167 @@ void PrefsDlg::setup_dictionary_font_page()
 	g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (on_setup_dictionary_font_button_clicked), this);
 }
 
+void PrefsDlg::on_setup_dictionary_cache_CreateCacheFile_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
+{
+	gboolean enable = gtk_toggle_button_get_active(button);
+	conf->set_bool("/apps/stardict/preferences/dictionary/create_cache_file",enable);
+}
+
+void PrefsDlg::on_setup_dictionary_cache_EnableCollation_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
+{
+	gboolean enable = gtk_toggle_button_get_active(button);
+	gtk_widget_set_sensitive(oPrefsDlg->collation_hbox, enable);
+	conf->set_bool("/apps/stardict/preferences/dictionary/enable_collation",enable);
+}
+
+void PrefsDlg::on_setup_dictionary_collation_optionmenu_changed(GtkOptionMenu *option_menu, PrefsDlg *oPrefsDlg)
+{
+	gint key = gtk_option_menu_get_history(option_menu);
+	conf->set_int("/apps/stardict/preferences/dictionary/collate_function", key);
+}
+
+static void clean_dir(const gchar *dirname)
+{
+	GDir *dir = g_dir_open(dirname, 0, NULL);
+	if (dir) {
+		const gchar *filename;
+		gchar fullfilename[256];
+		while ((filename = g_dir_read_name(dir))!=NULL) {
+			sprintf(fullfilename, "%s" G_DIR_SEPARATOR_S "%s", dirname, filename);
+			if (g_file_test(fullfilename, G_FILE_TEST_IS_DIR)) {
+				clean_dir(fullfilename);
+			} else if (g_str_has_suffix(filename,".oft") || g_str_has_suffix(filename,".clt")) {
+				g_unlink(fullfilename);
+			}
+		}
+		g_dir_close(dir);
+	}
+}
+
+void PrefsDlg::on_setup_dictionary_cache_cleanbutton_clicked(GtkWidget *widget, PrefsDlg *oPrefsDlg)
+{
+	std::string dirname = gStarDictDataDir+ G_DIR_SEPARATOR_S "dic";
+	clean_dir(dirname.c_str());
+	dirname = g_get_user_cache_dir();
+	dirname += G_DIR_SEPARATOR_S "stardict";
+	clean_dir(dirname.c_str());
+	g_rmdir(dirname.c_str());
+}
+
+void PrefsDlg::setup_dictionary_cache_page()
+{
+	GtkWidget *vbox;
+	vbox = gtk_vbox_new(false,12);
+#ifdef CONFIG_GPE
+        gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+        GtkWidget *nb_label = gtk_label_new(_("Cache"));
+        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox, nb_label);
+#else
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,NULL);
+#endif
+	GtkWidget *vbox1;
+	vbox1 = gtk_vbox_new(false,6);
+	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false,6);
+	GtkWidget *hbox;
+	hbox = gtk_hbox_new(false,6);
+	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
+	GtkWidget *image;
+	image = gtk_image_new_from_stock(GTK_STOCK_HARDDISK, GTK_ICON_SIZE_LARGE_TOOLBAR);
+	gtk_box_pack_start(GTK_BOX(hbox),image,false,false,0);
+	GtkWidget *label;
+	label = gtk_label_new(NULL);
+	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\" size=\"x-large\">Cache</span>"));
+	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
+	GtkWidget *hseparator;
+	hseparator = gtk_hseparator_new();
+	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
+	
+	vbox1 = gtk_vbox_new(false, 6);
+	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false, 0);
+	GtkWidget *check_button;
+	check_button = gtk_check_button_new_with_mnemonic(_("Create c_ache files to speed up loading."));
+	bool enable = conf->get_bool("/apps/stardict/preferences/dictionary/create_cache_file");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), enable);
+	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_cache_CreateCacheFile_ckbutton_toggled), (gpointer)this);
+	gtk_box_pack_start(GTK_BOX(vbox1),check_button,false,false,0);
+	check_button = gtk_check_button_new_with_mnemonic(_("_Sort word list by collate function."));
+	enable = conf->get_bool("/apps/stardict/preferences/dictionary/enable_collation");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), enable);
+	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_cache_EnableCollation_ckbutton_toggled), (gpointer)this);
+	gtk_box_pack_start(GTK_BOX(vbox1),check_button,false,false,0);
+	collation_hbox = gtk_hbox_new(false,6);
+	gtk_box_pack_start(GTK_BOX(vbox1),collation_hbox,false,false,0);
+	label=gtk_label_new(NULL);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, .5);
+	gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), _("\tCollate _function:"));
+	gtk_box_pack_start(GTK_BOX(collation_hbox),label,false,false,0);
+	GtkWidget *option_menu = gtk_option_menu_new();
+	GtkWidget *menu = gtk_menu_new();
+	GtkWidget *menuitem;
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__general__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__unicode__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__bin");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__czech__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__danish__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__esperanto__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__estonian__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__hungarian__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__icelandic__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__latvian__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__lithuanian__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__persian__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__polish__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__roman__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__romanian__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__slovak__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__slovenian__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__spanish__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__spanish2__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__swedish__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	menuitem=gtk_menu_item_new_with_mnemonic("utf8__turkish__ci");
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), menu);
+	int collate_function = conf->get_int("/apps/stardict/preferences/dictionary/collate_function");
+	gtk_option_menu_set_history(GTK_OPTION_MENU(option_menu), collate_function);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), option_menu);
+	gtk_box_pack_start(GTK_BOX(collation_hbox), option_menu, FALSE, FALSE, 0);
+	g_signal_connect (G_OBJECT (option_menu), "changed", G_CALLBACK (on_setup_dictionary_collation_optionmenu_changed), this);
+	gtk_widget_set_sensitive(collation_hbox, enable);
+
+	label = gtk_label_new(_("After enabled collation, when load the dictionaries for the first time, it will take some time for sorting, please wait for a moment."));
+	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+	gtk_box_pack_start(GTK_BOX(vbox1),label,false,false,0);
+
+	hbox = gtk_hbox_new(false,6);
+	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
+	GtkWidget *button = gtk_button_new_with_mnemonic(_("C_lean all cache files"));
+	gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_BUTTON));
+	gtk_box_pack_end(GTK_BOX(hbox),button,false,false,0);
+	g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (on_setup_dictionary_cache_cleanbutton_clicked), this);
+}
+
+
+
 void PrefsDlg::on_setup_dictionary_export_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
 {
 	gboolean enable = gtk_toggle_button_get_active(button);
@@ -632,31 +798,102 @@ void PrefsDlg::setup_dictionary_sound_page()
 	hseparator = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
 	
-	GtkWidget *hbox1;
-	hbox1 = gtk_hbox_new(false,0);
-	gtk_box_pack_start(GTK_BOX(vbox),hbox1,false,false,0);
-	
+	vbox1 = gtk_vbox_new(false, 6);
+	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false, 0);
+
 	GtkWidget *check_button;
 	check_button = gtk_check_button_new_with_mnemonic(_("_Enable sound event."));
 	bool enable=
 		conf->get_bool("/apps/stardict/preferences/dictionary/enable_sound_event");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), enable);
 	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_sound_ckbutton_toggled), (gpointer)this);
-	gtk_box_pack_start(GTK_BOX(hbox1),check_button,false,false,0);
+	gtk_box_pack_start(GTK_BOX(vbox1),check_button,false,false,0);
 #if defined(CONFIG_GTK) || defined(CONFIG_GPE)
-  GtkWidget *hbox2 = gtk_hbox_new(FALSE, 6);
-  label=gtk_label_new(_("Command for playing wav files:"));
-  gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
-  GtkWidget *e = gtk_entry_new();
-  gtk_widget_set_size_request(e, 50, -1);
+	GtkWidget *hbox2 = gtk_hbox_new(FALSE, 6);
+	label=gtk_label_new(_("Command for playing wav files:"));
+	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
+	GtkWidget *e = gtk_entry_new();
+	gtk_widget_set_size_request(e, 50, -1);
 	const std::string &playcmd=
 		conf->get_string("/apps/stardict/preferences/dictionary/play_command");
-  gtk_entry_set_text(GTK_ENTRY(e), playcmd.c_str());
-  gtk_box_pack_start(GTK_BOX(hbox2), e, TRUE, TRUE, 0);
-  gtk_widget_set_sensitive(hbox2, enable);  
-  ePlayCommand=GTK_ENTRY(e);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
+	gtk_entry_set_text(GTK_ENTRY(e), playcmd.c_str());
+	gtk_box_pack_start(GTK_BOX(hbox2), e, TRUE, TRUE, 0);
+	gtk_widget_set_sensitive(hbox2, enable);  
+	ePlayCommand=GTK_ENTRY(e);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox2, FALSE, FALSE, 0);
 #endif
+
+	label = gtk_label_new(_("RealPeopleTTS search path:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0, .5);
+	gtk_box_pack_start(GTK_BOX(vbox1),label,false,false,0);
+	tts_textview = gtk_text_view_new();
+	gtk_widget_set_size_request(tts_textview, -1, 70);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(tts_textview), GTK_WRAP_CHAR);
+	const std::string &ttspath = conf->get_string("/apps/stardict/preferences/dictionary/tts_path");
+	GtkTextBuffer *text_view_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(tts_textview));
+	gtk_text_buffer_set_text(text_view_buffer, ttspath.c_str(), -1);
+	GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
+                                       GTK_SHADOW_ETCHED_IN);
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_container_add(GTK_CONTAINER(scrolled_window), tts_textview);
+	gtk_box_pack_start(GTK_BOX(vbox1),scrolled_window,false,false,0);
+}
+
+void PrefsDlg::on_setup_mainwin_searchWhileTyping_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
+{
+  conf->set_bool("/apps/stardict/preferences/main_window/search_while_typing", 
+								 gtk_toggle_button_get_active(button));
+}
+
+void PrefsDlg::on_setup_mainwin_showfirstWhenNotfound_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
+{
+  conf->set_bool("/apps/stardict/preferences/main_window/showfirst_when_notfound", 
+								 gtk_toggle_button_get_active(button));
+}
+
+void PrefsDlg::setup_mainwin_input_page()
+{
+	GtkWidget *vbox;
+	vbox = gtk_vbox_new(false,12);
+#ifdef CONFIG_GPE
+        gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+        GtkWidget *nb_label = gtk_label_new(_("Input"));
+        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox, nb_label);
+#else
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,NULL);
+#endif
+	GtkWidget *vbox1;
+	vbox1 = gtk_vbox_new(false,6);
+	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false,6);
+	GtkWidget *hbox;
+	hbox = gtk_hbox_new(false,6);
+	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
+	GtkWidget *image;
+	image = gtk_image_new_from_stock(GTK_STOCK_EDIT,GTK_ICON_SIZE_LARGE_TOOLBAR);
+	gtk_box_pack_start(GTK_BOX(hbox),image,false,false,0);
+	GtkWidget *label;
+	label = gtk_label_new(NULL);
+	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\" size=\"x-large\">Input</span>"));
+	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
+	GtkWidget *hseparator;
+	hseparator = gtk_hseparator_new();
+	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
+
+	vbox1 = gtk_vbox_new(false, 6);
+	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false, 0);	
+	GtkWidget *check_button;
+	check_button = gtk_check_button_new_with_mnemonic(_("_Search while typing."));
+	gtk_box_pack_start(GTK_BOX(vbox1), check_button, FALSE, FALSE, 0);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), conf->get_bool("/apps/stardict/preferences/main_window/search_while_typing"));
+	g_signal_connect(G_OBJECT(check_button), "toggled", 
+									 G_CALLBACK(on_setup_mainwin_searchWhileTyping_ckbutton_toggled), this);
+	check_button = gtk_check_button_new_with_mnemonic(_("Show the _first word when not found."));
+	gtk_box_pack_start(GTK_BOX(vbox1),check_button,false,false,0);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), conf->get_bool("/apps/stardict/preferences/main_window/showfirst_when_notfound"));
+	g_signal_connect(G_OBJECT(check_button), "toggled", 
+									 G_CALLBACK(on_setup_mainwin_showfirstWhenNotfound_ckbutton_toggled), this);
 }
 
 void PrefsDlg::on_setup_mainwin_startup_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
@@ -666,6 +903,27 @@ void PrefsDlg::on_setup_mainwin_startup_ckbutton_toggled(GtkToggleButton *button
 }
 
 #ifdef _WIN32
+void PrefsDlg::on_setup_mainwin_autorun_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
+{
+	gboolean b = gtk_toggle_button_get_active(button);
+	HKEY hKEY;
+	LONG lRet;
+	if (b) {
+		lRet =RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Run",0,KEY_ALL_ACCESS,&hKEY);
+		if(lRet==ERROR_SUCCESS) {
+			std::string path = gStarDictDataDir+ G_DIR_SEPARATOR_S "stardict.exe";
+			RegSetValueEx(hKEY, "StarDict", 0, REG_SZ, (const BYTE*)path.c_str(), path.length()+1);
+			RegCloseKey(hKEY);
+		}
+	} else {
+		lRet =RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Run",0,KEY_ALL_ACCESS,&hKEY);
+		if(lRet==ERROR_SUCCESS) {
+			RegDeleteValue(hKEY, "StarDict");
+			RegCloseKey(hKEY);
+		}
+	}
+}
+
 void PrefsDlg::on_setup_mainwin_use_mainwindow_hotkey_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
 {
 	gboolean b = gtk_toggle_button_get_active(button);
@@ -707,7 +965,39 @@ void PrefsDlg::setup_mainwin_options_page()
 
 	vbox1 = gtk_vbox_new(false, 6);
 	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false, 0);	
+
 	GtkWidget *check_button;
+#ifdef _WIN32
+	check_button = gtk_check_button_new_with_mnemonic(_("_Auto run StarDict after boot."));
+	gtk_box_pack_start(GTK_BOX(vbox1),check_button,false,false,0);
+	gboolean autorun;
+
+	HKEY hKEY;
+	LONG lRet =RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Run",0,KEY_QUERY_VALUE,&hKEY);
+	if(lRet!=ERROR_SUCCESS) {
+		autorun = false;
+	} else {
+		char owner_Get[80];
+		DWORD cbData_1=80;
+		DWORD type_1=REG_SZ;
+		lRet=RegQueryValueEx(hKEY,"StarDict",NULL,&type_1,(LPBYTE)owner_Get,&cbData_1);
+		RegCloseKey(hKEY);
+		if((lRet!=ERROR_SUCCESS)||(cbData_1 > 80)) {
+			autorun = false;
+		} else {
+			std::string path = gStarDictDataDir+ G_DIR_SEPARATOR_S "stardict.exe";
+			if (strcmp(path.c_str(), owner_Get)==0)
+				autorun = true;
+			else
+				autorun = false;
+		}
+	}
+
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), autorun);
+	g_signal_connect(G_OBJECT(check_button), "toggled", 
+									 G_CALLBACK(on_setup_mainwin_autorun_ckbutton_toggled), this);
+#endif
+
 	check_button = gtk_check_button_new_with_mnemonic(_("Hide main window when _starting StarDict."));
 	gtk_box_pack_start(GTK_BOX(vbox1), check_button, FALSE, FALSE, 0);
 	bool hide=
@@ -1247,10 +1537,11 @@ void PrefsDlg::setup_floatwin_size_page()
 	hseparator = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
 	
-	GtkWidget *vbox2;
-	vbox2 = gtk_vbox_new(FALSE, 6);
-	gtk_box_pack_start(GTK_BOX(vbox),vbox2,false,false,0);
-	
+	GtkWidget *table;
+	table = gtk_table_new(3, 2, FALSE);
+	gtk_table_set_row_spacings(GTK_TABLE(table), 6);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 6);
+	gtk_box_pack_start(GTK_BOX(vbox),table,false,false,0);
 	
 	int max_width=
 		conf->get_int("/apps/stardict/preferences/floating_window/max_window_width");
@@ -1261,11 +1552,9 @@ void PrefsDlg::setup_floatwin_size_page()
 	gint screen_width = gdk_screen_get_width(screen);
 	gint screen_height = gdk_screen_get_height(screen);
 
-
-	GtkWidget *hbox1 = gtk_hbox_new(FALSE, 6);
 	label=gtk_label_new(NULL);
 	gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), _("Max window _width:"));
-	gtk_box_pack_start(GTK_BOX(hbox1),label,false,false,0);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	GtkWidget *spin_button;
 	spin_button = gtk_spin_button_new_with_range(MIN_MAX_FLOATWIN_WIDTH,screen_width,1);
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), spin_button);
@@ -1273,25 +1562,21 @@ void PrefsDlg::setup_floatwin_size_page()
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_button), max_width);
 	g_signal_connect(G_OBJECT(spin_button), "value-changed", 
 									 G_CALLBACK(on_setup_floatwin_size_max_width_spinbutton_changed), this);
-	gtk_box_pack_start(GTK_BOX(hbox1),spin_button,false,false,0);
+	gtk_table_attach(GTK_TABLE(table), spin_button, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	label=gtk_label_new(_("(default:320)"));
-	gtk_box_pack_start(GTK_BOX(hbox1),label,false,false,0);
-	gtk_box_pack_start(GTK_BOX(vbox2),hbox1,false,false,0);
+	gtk_table_attach(GTK_TABLE(table), label, 2, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 
-	GtkWidget *hbox2;
-	hbox2= gtk_hbox_new(false,6);
 	label=gtk_label_new(NULL);
 	gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), _("Max window hei_ght:"));
-	gtk_box_pack_start(GTK_BOX(hbox2),label,false,false,0);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 	spin_button = gtk_spin_button_new_with_range(MIN_MAX_FLOATWIN_HEIGHT,screen_height,1);
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), spin_button);
 	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin_button), GTK_UPDATE_IF_VALID);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_button), max_height);
 	g_signal_connect (G_OBJECT (spin_button), "value-changed", G_CALLBACK (on_setup_floatwin_size_max_height_spinbutton_changed), (gpointer)this);
-	gtk_box_pack_start(GTK_BOX(hbox2),spin_button,false,false,0);
+	gtk_table_attach(GTK_TABLE(table), spin_button, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 	label=gtk_label_new(_("(default:240)"));
-	gtk_box_pack_start(GTK_BOX(hbox2),label,false,false,0);
-	gtk_box_pack_start(GTK_BOX(vbox2),hbox2,false,false,0);
+	gtk_table_attach(GTK_TABLE(table), label, 2, 3, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 }
 #endif
 
@@ -1308,8 +1593,10 @@ GtkWidget* PrefsDlg::create_notebook ()
 #endif	
 	setup_dictionary_scan_page ();
 	setup_dictionary_font_page ();
+	setup_dictionary_cache_page ();
 	setup_dictionary_export_page ();
 	setup_dictionary_sound_page ();
+	setup_mainwin_input_page ();
 	setup_mainwin_options_page ();
 	setup_mainwin_searchwebsite_page();
 	setup_NotificationAreaIcon_options_page();
@@ -1335,7 +1622,7 @@ PrefsDlg::PrefsDlg(GtkWindow *parent, GdkPixbuf *logo, const std::list<std::stri
   window = NULL;  
 }
 
-void PrefsDlg::ShowModal(void)
+bool PrefsDlg::ShowModal()
 {		
   window = gtk_dialog_new();
   gtk_window_set_transient_for(GTK_WINDOW(window), parent_window);
@@ -1387,26 +1674,42 @@ void PrefsDlg::ShowModal(void)
   resize_categories_tree();
 #endif
   
-  while (gtk_dialog_run(GTK_DIALOG(window))==GTK_RESPONSE_HELP)
-    ;
-  const gchar *ch;
-  ch = gtk_entry_get_text(eExportFile);
-  if (ch[0])
-    conf->set_string("/apps/stardict/preferences/dictionary/export_file", ch);
+	gint result;
+	while ((result = gtk_dialog_run(GTK_DIALOG(window)))==GTK_RESPONSE_HELP)
+		;
+	if (result != GTK_RESPONSE_NONE) {
+		const gchar *ch;
+		ch = gtk_entry_get_text(eExportFile);
+		if (ch[0])
+			conf->set_string("/apps/stardict/preferences/dictionary/export_file", ch);
 #if defined(CONFIG_GTK) || defined(CONFIG_GPE)
-   ch = gtk_entry_get_text(ePlayCommand);
-   if (ch[0])
-     conf->set_string("/apps/stardict/preferences/dictionary/play_command", ch);
+		ch = gtk_entry_get_text(ePlayCommand);
+		if (ch[0])
+			conf->set_string("/apps/stardict/preferences/dictionary/play_command", ch);
 #endif
-  gtk_widget_hide(GTK_WIDGET(window));
-  gtk_widget_destroy(GTK_WIDGET(window));
-  window = NULL;
+		GtkTextBuffer *text_view_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(tts_textview));
+		GtkTextIter start_iter;
+		GtkTextIter end_iter;
+		gtk_text_buffer_get_start_iter(text_view_buffer, &start_iter);
+		gtk_text_buffer_get_end_iter(text_view_buffer, &end_iter);
+		gchar *text = gtk_text_buffer_get_text(text_view_buffer, &start_iter, &end_iter, FALSE);
+		conf->set_string("/apps/stardict/preferences/dictionary/tts_path", text);
+		gpAppFrame->oReadWord.loadpath(text);
+		g_free(text);
+		gtk_widget_destroy(GTK_WIDGET(window));
+		window = NULL;
+		return false;
+	} else {
+		return true;
+	}
 }
 
-void PrefsDlg::ShowModal(GtkWindow *parent, GdkPixbuf *logo, const std::list<std::string>& key_combs)
+void PrefsDlg::Close()
 {
-  PrefsDlg prefs_dlg(parent, logo, key_combs);
-  prefs_dlg.ShowModal();
+	if (window) {
+		gtk_widget_destroy (window);
+		window = NULL;
+	}
 }
 
 #ifndef CONFIG_GPE
