@@ -17,7 +17,7 @@ class AppCore;
 class AppFrame;
 
 #include "conf.h"
-#include "lib.h"
+#include "lib/lib.h"
 #include "skin.h"
 #include "mainwin.h"
 #ifdef _WIN32
@@ -31,6 +31,7 @@ class AppFrame;
 #include "floatwin.h"
 #include "selection.h"
 #include "dictmanagedlg.h"
+#include "prefsdlg.h"
 #include "readword.h"
 #include "iskeyspressed.hpp"
 
@@ -47,13 +48,15 @@ const int LIST_WIN_ROW_NUM = 30; //how many words show in the list win.
 class AppCore {
 private:
 	DictManageDlg *dict_manage_dlg;
+	PrefsDlg *prefs_dlg;
 
 	static int MatchWordCompare(const void * s1, const void * s2);
 	static gboolean on_delete_event(GtkWidget * window, GdkEvent *event , AppCore *oAppCore);
 	static gboolean on_window_state_event(GtkWidget * window, GdkEventWindowState *event , AppCore *oAppCore);
 	static gboolean vKeyPressReleaseCallback(GtkWidget * window, GdkEventKey *event , AppCore *oAppCore);
+	void reload_dicts();
 public:
-	glong *iCurrentIndex;
+	CurrentIndex *iCurrentIndex;
 	GtkWidget *window;
 	GtkTooltips *tooltips;
 
@@ -80,23 +83,25 @@ public:
 	void Create(gchar *queryword);
 	void End();
 	void Query(const gchar *word);
+	void BuildResultData(const char* sWord, CurrentIndex *iIndex, const gchar *piIndexValidStr, int iLib, gchar ***pppWord, gchar ****ppppWordData, bool &bFound, gint Method);
+	void FreeResultData(gchar ***pppWord, gchar ****ppppWordData);
 	bool SimpleLookupToFloat(const gchar* sWord, bool bShowIfNotFound);
 #ifdef _WIN32
 	bool SmartLookupToFloat(const gchar* sWord, int BeginPos, bool bShowIfNotFound);
 #endif
-	bool SimpleLookupToTextWin(const gchar* sWord,glong* piIndex, bool piIndexValid = false, bool bTryMoreIfNotFound = false);
+	bool SimpleLookupToTextWin(const gchar* sWord, CurrentIndex* piIndex, const gchar *piIndexValidStr = NULL, bool bTryMoreIfNotFound = false, bool bShowNotfound = true, bool isShowFirst = false);
 	void LookupDataToMainWin(const gchar *sWord);
 	void LookupWithFuzzyToMainWin(const gchar* word);
 	void LookupWithFuzzyToFloatWin(const gchar * word);
 	void LookupWithRuleToMainWin(const gchar* word);
-	void ShowDataToTextWin(const gchar **ppWord, gchar ** ppWordData,const gchar * sOriginWord);
+	void ShowDataToTextWin(gchar ***pppWord, gchar ****ppppWordData,const gchar * sOriginWord, bool isShowFirst);
 	void ShowTreeDictDataToTextWin(guint32 offset, guint32 size, gint iTreeDict);
 	void ShowNotFoundToTextWin(const char* sWord,const char* sReason, TextWinQueryResult query_result);
 	void ShowNotFoundToFloatWin(const char* sWord,const char* sReason, gboolean fuzzy);
 
 	void TopWinEnterWord(const gchar *text);
 	void TopWinWordChange(const gchar* sWord);
-	void ListWords(glong * iStartIndex);
+	void ListWords(const gchar *sWord, CurrentIndex* iStartIndex, bool showfirst);
 	
 	void ListClick(const gchar *word);
 	void PopupPrefsDlg();
