@@ -106,7 +106,7 @@ LRESULT CALLBACK DockLet::systray_mainmsg_handler(HWND hwnd, UINT msg, WPARAM wp
 {
 	static UINT taskbarRestartMsg; /* static here means value is kept across multiple calls to this func */
 
-  switch (msg) {
+	switch (msg) {
 	case WM_CREATE:
 		taskbarRestartMsg = RegisterWindowMessage("TaskbarCreated");
 		break;
@@ -115,9 +115,9 @@ LRESULT CALLBACK DockLet::systray_mainmsg_handler(HWND hwnd, UINT msg, WPARAM wp
 		switch(LOWORD(wparam)) {
 		case SYSTRAY_CMND_MENU_SCAN:
 			if (GetMenuState(gpAppFrame->oDockLet.systray_menu, SYSTRAY_CMND_MENU_SCAN, MF_BYCOMMAND) & MF_CHECKED) {
-	conf->set_bool("/apps/stardict/preferences/dictionary/scan_selection", FALSE);
-      }	else {
-	conf->set_bool("/apps/stardict/preferences/dictionary/scan_selection", TRUE);
+				conf->set_bool("/apps/stardict/preferences/dictionary/scan_selection", FALSE);
+			} else {
+				conf->set_bool("/apps/stardict/preferences/dictionary/scan_selection", TRUE);
 			}				
 			break;
 		case SYSTRAY_CMND_MENU_QUIT:
@@ -129,32 +129,39 @@ LRESULT CALLBACK DockLet::systray_mainmsg_handler(HWND hwnd, UINT msg, WPARAM wp
 	{
 		if ( lparam == WM_LBUTTONDOWN ) {
 			if (GetKeyState(VK_CONTROL)<0) {
-	conf->set_bool("/apps/stardict/preferences/dictionary/scan_selection", !conf->get_bool("/apps/stardict/preferences/dictionary/scan_selection"));
-		}
-    } else if ( lparam == WM_LBUTTONDBLCLK ) {
+				conf->set_bool("/apps/stardict/preferences/dictionary/scan_selection", !conf->get_bool("/apps/stardict/preferences/dictionary/scan_selection"));
+			}
+		} else if ( lparam == WM_LBUTTONDBLCLK ) {
 			// Only use left button will conflict with the menu.
 			if (GTK_WIDGET_VISIBLE(gpAppFrame->window)) {
-				stardict_systray_minimize(gpAppFrame->window);
-				gtk_widget_hide(gpAppFrame->window);
-      }	else {
+				HWND main_window = static_cast<HWND>(GDK_WINDOW_HWND(gpAppFrame->window->window));
+				if (IsIconic(main_window)) {
+					ShowWindow(main_window,SW_RESTORE);
+				//} else if (GetForegroundWindow() != main_window) {
+					//SetForegroundWindow(main_window);
+				} else {
+					stardict_systray_minimize(gpAppFrame->window);
+					gtk_widget_hide(gpAppFrame->window);
+				}
+			} else {
 				stardict_systray_maximize(gpAppFrame->window);
 				gtk_window_present(GTK_WINDOW(gpAppFrame->window));
 				if (gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(gpAppFrame->oTopWin.WordCombo)->entry))[0]) {
 					gtk_widget_grab_focus(gpAppFrame->oMidWin.oTextWin.view->Widget()); //so user can input word directly.
-	} else {
+				} else {
 					gtk_widget_grab_focus(GTK_COMBO(gpAppFrame->oTopWin.WordCombo)->entry); //this won't change selection text.
 				}
 			}
-    } else if (lparam == WM_MBUTTONDOWN) {
-      if (conf->get_bool("/apps/stardict/preferences/notification_area_icon/query_in_floatwin")) {
+		} else if (lparam == WM_MBUTTONDOWN) {
+			if (conf->get_bool("/apps/stardict/preferences/notification_area_icon/query_in_floatwin")) {
 				gpAppFrame->oSelection.LastClipWord.clear();
 				gtk_selection_convert (gpAppFrame->oSelection.selection_widget, GDK_SELECTION_PRIMARY, gpAppFrame->oSelection.UTF8_STRING_Atom, GDK_CURRENT_TIME);
-      } else {
+			} else {
 				stardict_systray_maximize(gpAppFrame->window);
 				gtk_window_present(GTK_WINDOW(gpAppFrame->window));
 				gtk_selection_convert (gpAppFrame->oMidWin.oTextWin.view->Widget(), GDK_SELECTION_PRIMARY, gpAppFrame->oSelection.UTF8_STRING_Atom, GDK_CURRENT_TIME);
 			}	
-    } else if (lparam == WM_RBUTTONUP) {
+		} else if (lparam == WM_RBUTTONUP) {
 			/* Right Click */
 			POINT mpoint;
 			GetCursorPos(&mpoint);
