@@ -6,7 +6,7 @@
  *  Description: Gaim systray functionality
  */
 
-// come from gaim. changed by Hu Zheng <huzheng_001@163.com> for StarDict.
+// Come from gaim. changed by Hu Zheng <huzheng_001@163.com> for StarDict.
 // http://stardict.sourceforge.net 2003.09.22
 
 #ifndef _SYSTRAY_H_
@@ -23,7 +23,6 @@ enum DockLetIconType {
 
 class DockLet{
 public:	
-	gboolean query_in_floatwin;
 	DockLet();
 	void init();
 	void cleanup();
