@@ -16,10 +16,10 @@ int main(int argc, char *argv[])
 	
 	dirs.push_back("./");
 	
-	Libs libs;
+	Libs libs(NULL, false, false, 0);
 	libs.load(dirs, List(), List());
 	std::vector<gchar *> reslist[libs.ndicts()];
-	if (libs.LookupData("letter", reslist)) 
+	if (libs.LookupData("letter", reslist, NULL, NULL, NULL)) 
 		return EXIT_SUCCESS;
 	else
 		return EXIT_FAILURE;
